import Image
import logging
import os
import re
import tempfile
import twitter
import urllib2
from django.db import models
from django.conf import settings
from django.contrib.auth.models import BaseUserManager, AbstractBaseUser, PermissionsMixin
from django.core.urlresolvers import reverse
from django.core.validators import validate_email
from django.template import Context
from django.template.loader import get_template
from django.utils.translation import ugettext as _
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from yapi.utils import generate_key
from yutils.email import EmailMessage

from exceptions import InvalidParameter
from utils import ConfirmationToken
from django.core import files

# Instantiate logger.
logger = logging.getLogger(__name__)


# Supported user credentials for authentication.
CREDENTIAL_TYPES = (
    ('email', 'E-mail'),
    ('facebook', 'Facebook'),
    ('twitter', 'Twitter')
)


class UserManager(BaseUserManager):
    """
    Since the custom User model defines different fields than Django's default, this custom
    manager must be implemented.
    """
    
    def create_user(self, name, email, password=None):
        """
        Creates and saves a User.
        """
        user = self.model(name=name, email=email)
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, name, email, password):
        """
        Creates and saves a superuser.
        """
        user = self.create_user(name, email, password=password)
        user.is_active = True
        user.is_staff = True
        user.is_superuser = True
        user.save()
        return user


class User(AbstractBaseUser, PermissionsMixin):
    """
    Custom User model.
    """    
    # Fields.
    email = models.EmailField(verbose_name=_("Email address"), max_length=100, unique=True, db_index=True)
    name = models.CharField(verbose_name=_("Name"), max_length=100)
    is_active = models.BooleanField(verbose_name=_("Active"), help_text=_("Designates whether this user should be treated as active. Unselect this instead of deleting accounts."), default=False)
    is_staff = models.BooleanField(verbose_name=_("Staff status"), help_text=_("Designates whether the user can log into this admin site."), default=False)
    created_at = models.DateTimeField(verbose_name=_("Created at"), auto_now_add=True)
    created_via = models.CharField(max_length=20, choices=CREDENTIAL_TYPES, blank=True)

    # Required for custom User models.
    objects = UserManager()

    # The name of the field on the User model that is used as the unique identifier.
    USERNAME_FIELD = 'email'
    
    # A list of the field names that must be provided when creating a user via the createsuperuser management command.
    REQUIRED_FIELDS = ['name']

    def get_full_name(self):
        """
        A longer formal identifier for the user.
        """
        return self.name

    def get_short_name(self):
        """
        A short, informal identifier for the user.
        """
        return self.name

    def __unicode__(self):
        """
        String representation of the model instance.
        """
        return self.email
    
    def pending_activation(self):
        """
        Returns whether the account is pending authentication or not.
        """
        # The account is considered to be pending authentication if there is an
        # 'unused' activation key.
        try:
            ActivationKey.objects.get(user=self, activated_at__isnull=True)
            return True
        except ObjectDoesNotExist:
            return False
        
    @staticmethod
    def new(name, email, password, credentials_type):
        """
        Creates a new account.
        """
        # Validate email address.
        try:
            validate_email(email)
        except ValidationError:
            raise InvalidParameter('email', _("Please provide a valid email address"))
        
        # Check if account with given email already exists.
        try:
            User.objects.get(email=email)
            # If this place is reached, then account with given email already exists. Abort.
            raise InvalidParameter('email', _("This email is already registered."))
        except ObjectDoesNotExist:
            pass
        
        # All check, create account.
        user = User.objects.create_user(name=name, email=email, password=password)
        user.created_via = credentials_type
        user.save()
        
        # Create activation key.
        try:
            ActivationKey.new(user)
        
        # Unable to create activation key, abort account creation.
        except:
            logger.error('Unable to create activation key! Name: ' + name + ', Email: ' + email, exc_info=1)
            user.delete()
            raise
        
        # Return.
        return user
    
    def update(self, name=None, password=None, email=None):
        """
        Update user account.
        """
        if name and name != '':
            self.name = name
        if password and password != '':
            self.set_password(password)
        if email and email != '':
            self.email = email
        self.save()
        
    def get_photo_url(self):
        """
        Returns the account's photo URL.
        """
        # If user has set a photo.
        if hasattr(self, 'userphoto'):
            photo_url = self.userphoto.file.url
    
        # Else, return default avatar.
        else:
            photo_url = settings.HOST_URL + settings.STATIC_URL + 'yaccounts/images/avatar_default.png'
            
        # Return.
        return photo_url
    
    def set_photo(self, photo_file, filename=None):
        """
        Sets the account photo.
        """    
        # 1) If user has photo, delete old file.
        if hasattr(self, 'userphoto'):
            user_photo = self.userphoto
            os.remove(user_photo.file.path)
        # If not, create it.
        else:
            user_photo = UserPhoto(user=self)
            
        # 2) If a filename is provided, set it.
        if filename and filename != '':
            user_photo.file.save(filename, photo_file, save=False)
        # If not, just add photo.
        else:
            user_photo.file = photo_file
            
        # 3) Save.
        user_photo.save()
        
        # 4) Resize image. User photos are fixed to 140x140.
        try:
            im = Image.open(user_photo.file.path)
            im = im.resize((140, 140), Image.ANTIALIAS)
            im.save(user_photo.file.path, format='JPEG')
        except IOError:
            user_photo.destroy() # In order to not get stuck with an invalid image and, therefore, a nasty icon in the browser :P
            logger.error('Error resizing user photo! User: ' + str(self.email))
            raise
        
        # 5) Return.
        return user_photo
    
    def set_photo_from_url(self, image_url, filename=None):
        """
        Fetches file from given URL and sets it as the account's photo.
        """
        # Fetch file from URL.
        response = urllib2.urlopen(image_url)
        
        # Create a temporary file.
        lf = tempfile.NamedTemporaryFile()
        lf.write(response.read())
        
        # If no filename is provided, make it the last part of the URL.
        if not filename or filename == '':
            name = image_url.split('/')[-1]
        else:
            name = filename
        
        # Set the account photo.
        self.set_photo(photo_file=files.File(lf), filename=name)


class UserPhoto(models.Model):
    """
    User profile picture.
    """
    user = models.OneToOneField(settings.AUTH_USER_MODEL)
    file = models.ImageField(upload_to='yaccounts/profile_pictures/')
    
    def __unicode__(self):
        """
        String representation of the instance.
        """
        return str(self.user)
    
    def destroy(self):
        """
        Deletes instance AND file.
        """
        os.remove(self.file.path)
        self.delete()


class ActivationKey(models.Model):
    """
    Keys used to activate new accounts.
    """
    user = models.OneToOneField(settings.AUTH_USER_MODEL)
    key = models.CharField(max_length=200, unique=True)
    created_at = models.DateTimeField(auto_now_add=True)
    activated_at = models.DateTimeField(blank=True, null=True)
    
    def __unicode__(self):
        """
        String representation of the model instance.
        """
        return str(self.user)
    
    @staticmethod
    def new(user):
        """
        Creates an activation key for given user.
        """
        activation_key = ActivationKey(user=user, key=generate_key(salt=user.email))
        activation_key.save()
        activation_key.send_activation_link()
        return True
    
    def send_activation_link(self):
        """
        Sends the activation URL to the customers's email.
        """
        try:
            # Email variables.
            d = Context({
                'name': self.user.name,
                'activation_link': settings.HOST_URL + reverse('yaccounts:confirm') + '?t=' + ConfirmationToken.generate(self.user.email, ConfirmationToken.ACCOUNT_ACTIVATION, self.key)
            })
            
            # Render plaintext email template.
            plaintext = get_template('yaccounts/email/create_confirmation.txt')
            text_content = plaintext.render(d)
            
            # Render HTML email template.
            html = get_template('yaccounts/email/create_confirmation.html')
            html_content = html.render(d)
            
            # Email options.
            subject = _("New Account")
            from_email = settings.YACCOUNTS['email_from']
            to = [{ 'name': self.user.name, 'email': self.user.email }]
            
            # Build message and send.
            email = EmailMessage(sender=from_email,
                                recipients=to,
                                subject=subject,
                                text_content=text_content,
                                html_content=html_content,
                                tags=['Account Confirm'])
            result = email.send()
            
            # Check if email wasn't sent.
            if not result['sent']:
                logger.error('Account Confirmation Email Not Sent! Result: ' + str(result['result']))
                raise
        except:
            logger.error('Unable to send confirmation email', exc_info=1)
            raise
        
        # Return great success.
        return True


class AuthenticationLog(models.Model):
    """
    Logs successful and failed logins.
    """
    # Available account statuses.
    ACCOUNT_STATUS = (
        ('active', 'Active'),
        ('disabled', 'Disabled'),
        ('does_not_exist', 'Does Not Exist'),
        ('pending_activation', 'Pending Activation')
    )
    
    # Fields.
    date = models.DateTimeField(auto_now_add=True)
    email = models.CharField(max_length=100)
    valid_credentials = models.BooleanField(default=False)
    credentials_type = models.CharField(max_length=20, choices=CREDENTIAL_TYPES, blank=True)
    account_status = models.CharField(max_length=20, choices=ACCOUNT_STATUS)
    success = models.BooleanField(default=False)
    ip_address = models.CharField(max_length=50)
    metadata = models.TextField(blank=True) # Additional data on the subject.
    
    def __unicode__(self):
        """
        String representation of the model instance.
        """
        return self.email
    
    @staticmethod
    def new(email, valid_credentials, credentials_type, account_status, success, ip_address, metadata):
        """
        Logs an authentication event.
        """
        # Build authentication log.
        authlog = AuthenticationLog(email=email,
                                    valid_credentials=valid_credentials,
                                    credentials_type=credentials_type,
                                    account_status=account_status,
                                    success=success,
                                    ip_address=ip_address,
                                    metadata=metadata)
        
        # Shave and return.
        authlog.save()
        return authlog
    
    
class ResetRequest(models.Model):
    """
    Account password reset.
    """
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    key = models.CharField(max_length=200, unique=True)
    created_at = models.DateTimeField(auto_now_add=True)
    reset_at = models.DateTimeField(blank=True, null=True)
    
    def __unicode__(self):
        """
        String representation of the model instance.
        """
        return str(self.user)
    
    @staticmethod
    def new(user):
        """
        Creates an activation key for given user.
        """
        reset_request = ResetRequest(user=user, key=generate_key(salt=user.email))
        reset_request.save()
        try:
            reset_request.send_reset_link()
        except:
            reset_request.delete()
            raise
        return True
    
    def send_reset_link(self):
        """
        Sends the reset URL to the customers's email.
        """
        try:
            # Email variables.
            d = Context({
                'name': self.user.name,
                'reset_link': settings.HOST_URL + reverse('yaccounts:reset_confirm') + '?t=' + ConfirmationToken.generate(self.user.email, ConfirmationToken.ACCOUNT_PASSWORD_RESET, self.key)
            })
            
            # Render plaintext email template.
            plaintext = get_template('yaccounts/email/reset_confirmation.txt')
            text_content = plaintext.render(d)
            
            # Render HTML email template.
            html = get_template('yaccounts/email/reset_confirmation.html')
            html_content = html.render(d)
            
            # Email options.
            subject = _("Reset Password")
            from_email = settings.YACCOUNTS['email_from']
            to = [{ 'name': self.user.name, 'email': self.user.email }]
            
            # Build message and send.
            email = EmailMessage(sender=from_email,
                                recipients=to,
                                subject=subject,
                                text_content=text_content,
                                html_content=html_content,
                                tags=['Account Reset'])
            result = email.send()
            
            # Check if email wasn't sent.
            if not result['sent']:
                logger.error('Password Reset Email Not Sent! Result: ' + str(result['result']))
                raise
        except:
            logger.error('Unable to send password reset email', exc_info=1)
            raise
        
        # Return great success.
        return True
    

class EmailUpdate(models.Model):
    """
    Account email update request/validation.
    """
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    old_email = models.EmailField()
    new_email = models.EmailField()
    key = models.CharField(max_length=200, unique=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(blank=True, null=True)
    
    def __unicode__(self):
        """
        String representation of the model instance.
        """
        return str(self.user)
    
    @staticmethod
    def new(user, email):
        """
        Creates an email update request for given user.
        """
        email_update = EmailUpdate(user=user, old_email=user.email, new_email=email, key=generate_key(salt=(str(user.email)+str(email))))
        email_update.save()
        try:
            email_update.send_confirmation_link()
        except:
            email_update.delete()
            raise
        return True
    
    def send_confirmation_link(self):
        """
        Sends the confirmation URL to the customers's email.
        """
        try:
            # Email variables.
            d = Context({
                'name': self.user.name,
                'confirmation_link': settings.HOST_URL + reverse('yaccounts:confirm') + '?t=' + ConfirmationToken.generate(email=self.user.email,
                                                                                                                          operation=ConfirmationToken.ACCOUNT_EMAIL_UPDATE,
                                                                                                                          key=self.key,
                                                                                                                          more={ 'new_email': self.new_email })
            })
            
            # Render plaintext email template.
            plaintext = get_template('yaccounts/email/email_update_confirmation.txt')
            text_content = plaintext.render(d)
            
            # Render HTML email template.
            html = get_template('yaccounts/email/email_update_confirmation.html')
            html_content = html.render(d)
            
            # Email options.
            subject = _("Confirm Email")
            from_email = settings.YACCOUNTS['email_from']
            to = [{ 'name': self.user.name, 'email': self.new_email }]
            
            # Build message and send.
            email = EmailMessage(sender=from_email,
                                recipients=to,
                                subject=subject,
                                text_content=text_content,
                                html_content=html_content,
                                tags=['Account Email Update'])
            result = email.send()
            
            # Check if email wasn't sent.
            if not result['sent']:
                logger.error('Email Update Confirmation Not Sent! Result: ' + str(result['result']))
                raise
        except:
            logger.error('Unable to send email confirmation update link', exc_info=1)
            raise
        
        # Return great success.
        return True
    
    
class TwitterProfile(models.Model):
    """
    Twitter profiles connected to user accounts.
    """
    user = models.OneToOneField(settings.AUTH_USER_MODEL)
    twitter_user_id = models.CharField(max_length=200, unique=True, db_index=True)
    screen_name = models.CharField(max_length=50)
    access_token = models.CharField(max_length=200)
    access_token_secret = models.CharField(max_length=200)
    created_at = models.DateTimeField(auto_now_add=True)
    last_used = models.DateTimeField(blank=True, null=True)
    
    def __unicode__(self):
        """
        String representation of the model instance.
        """
        return str(self.user)
    
    @staticmethod
    def new(user, userinfo, access_token):
        """
        Links a Twitter profile with account.
        """
        # Create Twitter profile.
        twitter_profile = TwitterProfile(user=user,
                                         twitter_user_id=userinfo.id,
                                         screen_name=userinfo.screen_name,
                                         access_token=access_token['oauth_token'],
                                         access_token_secret=access_token['oauth_token_secret'])
        twitter_profile.save()
        
        # If user's account doesn't have a photo, fetch Twitter profile's photo.
        if not hasattr(twitter_profile.user, 'userphoto'):
            twitter_profile.user.set_photo_from_url(twitter_profile.get_photo_url())
        
        # Return.
        return twitter_profile
    
    def update(self, userinfo, access_token):
        """
        Updates a user's Twitter profile.
        """
        self.screen_name = userinfo.screen_name
        self.access_token = access_token['oauth_token']
        self.access_token_secret = access_token['oauth_token_secret']
        self.save()
        return self
    
    def get_photo_url(self):
        """
        Returns the profile's photo url.
        """
        # Initialize API connector stuffz.
        api = twitter.Api(consumer_key=settings.YACCOUNTS['twitter_oauth']['consumer_key'],
                          consumer_secret=settings.YACCOUNTS['twitter_oauth']['consumer_secret'],
                          access_token_key=self.access_token,
                          access_token_secret=self.access_token_secret)
        
        # Get profile image URL from userinfo.
        image_url = api.VerifyCredentials().profile_image_url
        
        # Since we want the original size of the image, we have to change the filename in the URL, by removing '_normal'.
        splited_url = image_url.split('/')
        splited_url[-1] = re.sub('_normal', '', splited_url[-1])
        
        # Return 'updated' URL.
        return '/'.join(splited_url)
    
    
class FacebookProfile(models.Model):
    """
    Holds a user's Fakebook profile.
    """
    user = models.OneToOneField(settings.AUTH_USER_MODEL)
    facebook_user_id = models.CharField(max_length=50, unique=True, db_index=True)
    name = models.CharField(max_length=50)
    access_token = models.CharField(max_length=200)
    created_at = models.DateTimeField(auto_now_add=True)
    last_used = models.DateTimeField(blank=True, null=True)

    def __unicode__(self):
        """
        String representation of the model instance.
        """
        return str(self.user)
    
    @staticmethod
    def new(user, userinfo, access_token):
        """
        Links a Facebook profile with account.
        """
        # Create Twitter profile.
        facebook_profile = FacebookProfile(user=user,
                                           facebook_user_id=userinfo.id,
                                           name=userinfo.name,
                                           access_token=access_token)
        facebook_profile.save()
        
        # If user's account doesn't have a photo, fetch Twitter profile's photo.
        if not hasattr(facebook_profile.user, 'userphoto'):
            facebook_profile.user.set_photo_from_url(image_url=facebook_profile.get_photo_url(), filename=str(facebook_profile.facebook_user_id) + '.png')
        
        # Return.
        return facebook_profile
    
    def update(self, userinfo, access_token):
        """
        Updates a user's Facebook profile.
        """
        self.name = userinfo.name
        self.access_token = access_token
        self.save()
        return self
    
    def get_photo_url(self):
        """
        Returns the profile's photo url.
        """
        return FacebookProfile.get_profile_image_url(self.facebook_user_id)
        
    @staticmethod
    def get_profile_image_url(fbid):
        """
        Returns the link to the profile image, given the user's Facebook ID.
        """
        return 'https://graph.facebook.com/' + fbid + '/picture?type=large'


from django.conf.urls import patterns, url

from handlers import AccountHandler, AccountPhotoHandler, ApiKeysHandler, ApiKeyIdHandler

urlpatterns = patterns('',
    url(r'^/?$', AccountHandler.as_view(), name='index'),
    url(r'^/photo/?$', AccountPhotoHandler.as_view(), name='photo'),
    url(r'^/api_keys/?$', ApiKeysHandler.as_view(), name='api_keys'),
    url(r'^/api_keys/(?P<pk>[0-9]+)/?$', ApiKeyIdHandler.as_view(), name='api_key_id')
)


import logging
from django import forms
from django.utils.translation import ugettext as _
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist

from models import EmailUpdate

# Instantiate logger.
logger = logging.getLogger(__name__)


class UserForm(forms.Form):
    """
    Form for updating User.
    """
    name = forms.CharField(max_length=100)
    email = forms.EmailField()
    password_old = forms.CharField(max_length=100, required=False)
    password_new = forms.CharField(max_length=100, required=False)
    password_confirm = forms.CharField(max_length=100, required=False)
    
    def __init__(self,*args,**kwargs):
        """
        Constructor.
        """
        self.user = kwargs.pop('user')
        super(UserForm,self).__init__(*args, **kwargs)
        
    def clean_email(self):
        """
        Add additional custom validation to field.
        """
        data = self.cleaned_data['email']
        
        # If email is different from the account's email address
        if self.user.email != data: 
            
            # 1) Check if there isn't a pending update request.
            if EmailUpdate.objects.filter(user=self.user, updated_at__isnull=True):
                raise forms.ValidationError('You already requested an email update. Verify or cancel.')
            
            # 2) If not, check if the new email isn't already registered.
            try:
                get_user_model().objects.get(email=data)
                raise forms.ValidationError('Email already registered.') 
            
            # If this place is reached, then email is different and not already registered with another account.
            except ObjectDoesNotExist:
                
                # Send email update confirmation to new email.
                try:
                    EmailUpdate.new(user=self.user, email=data)
                except:
                    logger.error('Unable to send email update confirmation! User: ' + self.user.email + ', Email Update: ' + data, exc_info=1)
                    raise forms.ValidationError('Unable to send email update confirmation email.')
                
                # Return.
                return data
    
    def clean(self):
        """
        Override default method to add additional validation.
        """
        # Superclass stuffz.
        cleaned_data = super(UserForm, self).clean()
        
        # New password provided.
        if cleaned_data.get('password_new'):
            
            # 1) Old password must check.
            if not cleaned_data.get('password_old'):
                self._errors['password_old'] = self.error_class([_('This field is required.')])
                del cleaned_data['password_new']
            elif not self.user.check_password(cleaned_data.get('password_old')):
                    self._errors['password_old'] = self.error_class([_('Invalid password.')])
                    del cleaned_data['password_new']
                    
            # 2) Double check new password with confirmation.
            elif not cleaned_data.get('password_confirm'):
                self._errors['password_confirm'] = self.error_class([_('This field is required.')])
                del cleaned_data['password_new']
            elif cleaned_data.get('password_confirm') != cleaned_data.get('password_new'):
                self._errors['password_confirm'] = self.error_class([_('Confirmation password doesn\'t match.')])
                del cleaned_data['password_new']
        
        # Return.
        return cleaned_data


class UserPhotoForm(forms.Form):
    """
    Form for User account photo.
    """
    file = forms.ImageField()


import cgi
import datetime
import facebook
import json
import logging
import urllib
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login, get_user_model
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.core.validators import validate_email
from django.db.utils import IntegrityError
from django.http.response import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.utils.translation import ugettext as _
from yapi.utils import generate_key

from models import AuthenticationLog, FacebookProfile

# Instantiate logger.
logger = logging.getLogger(__name__)

# Application's OAuth settings.
app_id = settings.YACCOUNTS['facebook_oauth']['app_id']
app_secret = settings.YACCOUNTS['facebook_oauth']['app_secret']

# Facebook OAuth URLs.
authorize_url = 'https://graph.facebook.com/oauth/authorize'
access_token_url = 'https://graph.facebook.com/oauth/access_token'


class UserInfo:
    """
    Facebook API userinfo object wrapper.
    """
    def __init__(self, userinfo):
        self.id = userinfo['id']
        self.name = userinfo['name']


def login_request(request):
    """
    Starts Facebook authentication.
    """
    
    # Check if Facebook authentication is enabled.
    try:
        settings.YACCOUNTS['signup_available'].index('FACEBOOK')
    except ValueError:
        messages.add_message(request, messages.ERROR, _('Facebook login not available.'))
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    # If there is an URL to return when login finishes,
    # store it in session in order to make it acessible in
    # the facebook return method (would be better to pass it in the URLs?)
    request.session['login_next'] = request.GET.get('next', reverse('yaccounts:index'))
    
    # Start authentication redirecting to respective page.
    return HttpResponseRedirect(authorize_url + '?client_id=' + app_id + '&redirect_uri=' + settings.HOST_URL + reverse('yaccounts:facebook_return'))


def login_return(request):
    """
    OAuth callback.
    """
    
    ##################
    # 1) Validations #
    ##################
    
    # Check for required parameter.
    code = request.GET.get('code', None)
    if not code:
        messages.add_message(request, messages.ERROR, _('Facebook login error #1'))
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    ###########################
    # 2) Request access token #
    ###########################
        
    request_access_token_url = access_token_url + '?client_id=' + app_id \
                                 + '&redirect_uri=' + settings.HOST_URL + reverse('yaccounts:facebook_return') \
                                 + '&client_secret=' + app_secret \
                                 + '&code=' + code
    response = cgi.parse_qs(urllib.urlopen(request_access_token_url).read())
    try:
        access_token = response['access_token'][-1]
    except KeyError:
        messages.add_message(request, messages.ERROR, _('Facebook login error #2'))
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    # Verify credentials.
    # Validate user access token. If we can fetch it's information, the token is good!
    try:
        fb_api = facebook.GraphAPI(access_token=access_token)
        userinfo = UserInfo(fb_api.get_object('me'))
    except:
        messages.add_message(request, messages.ERROR, _('Facebook login error #3'))
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    #########################################################
    # 3) Authenticate user with given Facebook credentials. #
    #########################################################
    user = authenticate(facebook_userinfo=userinfo, facebook_access_token=access_token)
    
    ##
    # a) Facebook profile is linked with existing user account.
    if user:
        
        # Additional information on the request that should be logged.
        metadata = { 'user_agent': request.META['HTTP_USER_AGENT'] }
        
        # If user is active, login account and redirect to next page (if provided, else account profile)
        if user.is_active:
            
            # Create user login session.
            login(request, user)
            
            # Log authentication.
            AuthenticationLog.new(email=user.email,
                                  valid_credentials=True,
                                  credentials_type='facebook',
                                  account_status='active',
                                  success=True,
                                  ip_address=request.META['REMOTE_ADDR'],
                                  metadata=json.dumps(metadata))
            
            # Redirect to either page referenced as next or accounts index.
            return HttpResponseRedirect(request.session.get('login_next', reverse('yaccounts:index')))
        
        # User account is inactive.
        else:
            
            # Log authentication.
            AuthenticationLog.new(email=user.email,
                                  valid_credentials=True,
                                  credentials_type='facebook',
                                  account_status='disabled',
                                  success=False,
                                  ip_address=request.META['REMOTE_ADDR'],
                                  metadata=json.dumps(metadata))
            
            # Message.
            messages.warning(request, _("Your account is disabled."))
            
            # Return.
            return HttpResponseRedirect(reverse('yaccounts:login'))
        
    ##
    # b) Unknown Facebook profile.
    else:
        
        # i) If there is an account logged in, (attempt to) link the Facebook profile with it.
        if request.user.is_authenticated():
            try:
                FacebookProfile.new(user=request.user, userinfo=userinfo, access_token=access_token)
                messages.success(request, _("Facebook account connected successfully."))
            except IntegrityError:
                messages.error(request, _("You already have a Facebook profile linked to your account."))
            return HttpResponseRedirect(reverse('yaccounts:index'))
        
        # ii) Create new account.
        else:
            # Place Facebook info in a session variable in order for it to be accessed in the registration page.
            request.session['facebook_create'] = {
                    'facebook_user_id': userinfo.id,
                    'name': userinfo.name,
                    'profile_image_url': FacebookProfile.get_profile_image_url(userinfo.id),
                    'access_token': access_token,
                    'expires': (datetime.datetime.now() + datetime.timedelta(seconds=5*60)).strftime('%s') # Convert to epoch to be JSON serializable.
            }
            return HttpResponseRedirect(reverse('yaccounts:facebook_create'))
        
        
def create_account(request):
    """
    Create new account with Facebook credentials.
    """
    
    #
    # Lets validate if we should be here.
    #
    
    # If user is authenticated, then this place shouldn't be reached.
    # Facebook profile, if valid, should be linked with the logged in account and not create a new account.
    if request.user.is_authenticated():
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    # In order to create account with Facebook profile, its details should have been stored in session.
    facebook_create = request.session.get('facebook_create', None)
    if not facebook_create:
        messages.error(request, _('Facebook login error #4'))
        return HttpResponseRedirect(reverse('yaccounts:login'))
    
    # If time window for registration of new account with this Facebook profile has expired,
    # delete it from session and restart Facebook login process.
    if datetime.datetime.now() > datetime.datetime.fromtimestamp(float(facebook_create['expires'])):
        del request.session['facebook_create']
        return HttpResponseRedirect(reverse('yaccounts:facebook_login'))
    
    #
    # Proceed with account creation.
    #
    email = ''
    
    # A form was received.
    if request.method == 'POST':
        
        proceed = True
        
        # Fetch mandatory params.
        try:
            email = request.POST['email']
        except KeyError:
            proceed = False
            messages.error(request, _("Please provide an e-mail address."))
            
        # Validate email address.
        try:
            validate_email(email)
        except ValidationError:
            proceed = False
            messages.error(request, _("Please provide a valid email address."))
            
        # Check if Facebook profile is already connected to another account.
        try:
            FacebookProfile.objects.get(facebook_user_id=facebook_create['facebook_user_id'])
            proceed = False
            messages.error(request, _("Facebook profile already connected to another account."))
        except ObjectDoesNotExist:
            pass
        
        # Check if account exists with given email address.
        try:
            get_user_model().objects.get(email=email)
            proceed = False
            messages.error(request, _("Email already registered."))
        except ObjectDoesNotExist:
            pass
        
        #
        # Everything checks! \o/
        #
        if proceed:
            
            # 1) Create user with random password.
            try:
                random_password = generate_key(email + str(datetime.datetime.now()) + str(facebook_create['facebook_user_id']))
                random_password += datetime.datetime.now().strftime('%s')
                user = get_user_model().new(name=facebook_create['name'], email=email, password=random_password, credentials_type='facebook')
            except:
                logger.error('Error creating user via Facebook! #5 ' + str(facebook_create), exc_info=1)
                messages.error(request, _('Facebook login error #5'))
            
            # 2) Create Facebook profile and associate it with the new user.
            try:
                userinfo = UserInfo(userinfo={ 'id': facebook_create['facebook_user_id'], 'name': facebook_create['name'] })
                FacebookProfile.new(user=user, userinfo=userinfo, access_token=facebook_create['access_token'])
                # Redirect to login page with message.
                messages.success(request, _("An email was sent in order to confirm your account."))
                return HttpResponseRedirect(reverse('yaccounts:login'))
            except:
                user.delete() # Delete newly created user (as it would be inaccessible since the Facebook Profile wasn't created!)
                logger.error('Error creating user via Facebook! #6 ' + str(facebook_create), exc_info=1)
                messages.error(request, _('Facebook login error #6'))
    
    # Render page.
    return render_to_response('yaccounts/create_social.html',
                              { 'avatar': facebook_create['profile_image_url'],
                               'username': facebook_create['name'],
                               'email': email,
                               'post_url': reverse('yaccounts:facebook_create') },
                              context_instance=RequestContext(request))
    
    
@login_required
def disconnect_account(request):
    """
    Disconnects Facebook profile from user account.
    """
    
    # Check if user has a Facebook profile connected.
    if not hasattr(request.user, 'facebookprofile'):
        messages.error(request, _("You don't have a Facebook account connected."))
    
    # Disconnect!
    else:
        request.user.facebookprofile.delete()
        messages.success(request, _("Facebook account disconnected."))
    
    # Redirect to account page.
    return HttpResponseRedirect(reverse('yaccounts:index'))


from django import forms
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.forms import ReadOnlyPasswordHashField
from django.contrib.auth.models import Group
from django.utils.translation import ugettext as _

from models import User, UserPhoto, ActivationKey, AuthenticationLog, ResetRequest, EmailUpdate, TwitterProfile, FacebookProfile


class UserCreationForm(forms.ModelForm):
    """
    A form for creating new users. Includes all the required fields, plus a repeated password.
    """
    password1 = forms.CharField(label=_("Password"), widget=forms.PasswordInput)
    password2 = forms.CharField(label=_("Password confirmation"), widget=forms.PasswordInput)

    class Meta:
        model = User

    def clean_password2(self):
        """
        Check that the two password entries match.
        """
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(_("Passwords don't match"))
        return password2

    def save(self, commit=True):
        """
        Save the provided password in hashed format.
        """
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data['password1'])
        if commit:
            user.save()
        return user


class UserChangeForm(forms.ModelForm):
    """
    A form for updating users. Includes all the fields on
    the user, but replaces the password field with admin's
    password hash display field.
    """
    password = ReadOnlyPasswordHashField(label=_("Password"),
        help_text= (_("Raw passwords are not stored, so there is no way to see "
                    "this user's password, but you can change the password "
                    "using <a href=\"password/\">this form</a>.")))

    class Meta:
        model = User

    def clean_password(self):
        """
        Regardless of what the user provides, return the initial value.
        This is done here, rather than on the field, because the
        field does not have access to the initial value
        """
        return self.initial['password']


class UserAdmin(UserAdmin):
    """
    Django admin for custom User model.
    """
    # The forms to add and change user instances.
    form = UserChangeForm
    add_form = UserCreationForm

    # The fields to be used in displaying the User model.
    # These override the definitions on the base UserAdmin
    # that reference specific fields on auth.User.
    list_display = ('email', 'name', 'is_active', 'is_staff', 'is_superuser', 'created_at', 'created_via', 'last_login')
    list_filter = ('is_active', 'is_staff', 'is_superuser', 'created_via')
    fieldsets = (
        (None, { 'fields': ('name', 'email', 'password') }),
        #('Permissions', { 'fields': ('is_active', 'is_staff', 'is_superuser', 'groups', 'user_permissions') }), # We're not using Django authorization, yet.
        ('Permissions', { 'fields': ('is_active', 'is_staff', 'is_superuser') }),
        ('Important dates', { 'fields': ('last_login', 'created_at', 'created_via') })
    )
    add_fieldsets = (
        (None, { 'classes': ('wide',), 'fields': ('name', 'email', 'password1', 'password2') }),
    )
    search_fields = ('email',)
    ordering = ('email',)
    filter_horizontal = ('user_permissions',)
    readonly_fields = ('last_login', 'created_at', 'created_via')

# Register the custom User admin.
admin.site.register(User, UserAdmin)


class UserPhotoAdmin(admin.ModelAdmin):
    list_display = ('user', 'file')
admin.site.register(UserPhoto, UserPhotoAdmin)


class ActivationKeyAdmin(admin.ModelAdmin):
    list_display = ('user', 'key', 'created_at', 'activated_at')
    search_fields = ('user__email', 'key')
admin.site.register(ActivationKey, ActivationKeyAdmin)


class AuthenticationLogAdmin(admin.ModelAdmin):
    list_display = ('date', 'email', 'valid_credentials', 'credentials_type', 'account_status', 'ip_address', 'metadata', 'success')
    search_fields = ('email', 'ip_address')
    list_filter = ('valid_credentials', 'credentials_type', 'success', 'account_status')
admin.site.register(AuthenticationLog, AuthenticationLogAdmin)


class ResetRequestAdmin(admin.ModelAdmin):
    list_display = ('user', 'key', 'created_at', 'reset_at')
    search_fields = ('user__email', 'key')
admin.site.register(ResetRequest, ResetRequestAdmin)


class EmailUpdateAdmin(admin.ModelAdmin):
    list_display = ('user', 'old_email', 'new_email', 'key', 'created_at', 'updated_at')
    search_fields = ('user__email', 'old_email', 'new_email', 'key')
admin.site.register(EmailUpdate, EmailUpdateAdmin)


class TwitterProfileAdmin(admin.ModelAdmin):
    list_display = ('user', 'twitter_user_id', 'screen_name', 'created_at', 'last_used')
    search_fields = ('user__email', 'twitter_user_id', 'screen_name', 'access_token', 'access_token_secret')
admin.site.register(TwitterProfile, TwitterProfileAdmin)


class FacebookProfileAdmin(admin.ModelAdmin):
    list_display = ('user', 'facebook_user_id', 'name', 'created_at', 'last_used')
    search_fields = ('user__email', 'facebook_user_id', 'name', 'access_token')
admin.site.register(FacebookProfile, FacebookProfileAdmin)


# Unregister the Group model from admin (we're not using it, yet)
admin.site.unregister(Group)


class InvalidParameter(Exception):
    """
    Exception raised when there is a problem with a method parameter. 
    """
    def __init__(self, parameter, message):
        self.parameter = parameter
        self.message = message
    def __str__(self):
        return repr(self.err)


import datetime
import logging
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist

from models import ActivationKey, TwitterProfile, FacebookProfile

# Instantiate logger.
logger = logging.getLogger(__name__)


class ActivationKeyAuthenticationBackend(object):
    """
    Authentication backend that validates user given activation key.
    """
    
    def authenticate(self, email, activation_key):
        """
        Mandatory method for Authentication Backends, validates provided set of credentials.
        """
        # Key cannot be empty string.
        if activation_key == '':
            return None
        
        # Check if there is an account with given email address.
        try:
            user = get_user_model().objects.get(email=email)
        except get_user_model().DoesNotExist:
            return None
        
        # Check if there is an unused activation for that account with given activation key.
        try:
            activation_key = ActivationKey.objects.get(user=user, key=activation_key, activated_at__isnull=True)
        except ObjectDoesNotExist:
            return None
        
        # Mark activation key as used, by timestamping activation date.
        activation_key.activated_at = datetime.datetime.now()
        activation_key.save()
        
        # Activate account.
        user.is_active = True
        user.save()
        
        # Return user.
        return user

    def get_user(self, user_id):
        """
        Mandatory method for Authentication Backends.
        """
        try:
            return get_user_model().objects.get(pk=user_id)
        except get_user_model().DoesNotExist:
            return None
        
        
class TwitterBackend:
    """
    Custom backend authentication to enable Twitter OAuth login.
    """

    def authenticate(self, twitter_userinfo, twitter_access_token):
        """
        Mandatory method for Authentication Backends, validates provided set of credentials.
        """
        try:
            # Validate credentials (i.e. check if this Twitter profile is already registered)
            twitter_profile = TwitterProfile.objects.get(twitter_user_id=twitter_userinfo.id)
        
            # Always update Twitter account information that comes with OAuth access token
            # (user might have revoked access to the application / changed screen name / etc)
            twitter_profile.update(twitter_userinfo, twitter_access_token)
            
            # Update last used timestamp.
            twitter_profile.last_used = datetime.datetime.now()
            twitter_profile.save()
            
            # If we're here, these credentials are valid (an account exists that is connected to this Twitter profile)
            return twitter_profile.user
        
        # Twitter profile not registered.
        except ObjectDoesNotExist:
            return None
            
    def get_user(self, user_id):
        """
        Mandatory method for Authentication Backends.
        """
        try:
            return get_user_model().objects.get(pk=user_id)
        except get_user_model().DoesNotExist:
            return None
        
        
class FacebookBackend:
    """
    Custom backend authentication to enable Facebook OAuth login.
    """

    def authenticate(self, facebook_userinfo, facebook_access_token):
        """
        Mandatory method for Authentication Backends, validates provided set of credentials.
        """
        try:
            # Validate credentials (i.e. check if this Facebook profile is already registered)
            facebook_profile = FacebookProfile.objects.get(facebook_user_id=facebook_userinfo.id)
        
            # Always update Facebook account information that comes with OAuth access token
            # (user might have revoked access to the application / changed screen name / etc)
            facebook_profile.update(facebook_userinfo, facebook_access_token)
            
            # Update last used timestamp.
            facebook_profile.last_used = datetime.datetime.now()
            facebook_profile.save()
            
            # If we're here, these credentials are valid (an account exists that is connected to this Facebook profile)
            return facebook_profile.user
        
        # Facebook profile not registered.
        except ObjectDoesNotExist:
            return None
            
    def get_user(self, user_id):
        """
        Mandatory method for Authentication Backends.
        """
        try:
            return get_user_model().objects.get(pk=user_id)
        except get_user_model().DoesNotExist:
            return None


from django.conf.urls import patterns, url

import views
import views_facebook
import views_twitter

urlpatterns = patterns('',
                       
    # Account views.
    url(r'^/?$', views.index, name='index'),
    url(r'^/login/?$', views.login_account, name='login'),
    url(r'^/logout/?$', views.logout_account, name='logout'),
    url(r'^/create/?$', views.create_account, name='create'),
    url(r'^/confirm/?$', views.confirm_operation, name='confirm'),
    url(r'^/reset/?$', views.reset_account, name='reset'),
    url(r'^/reset/confirm/?$', views.reset_confirm, name='reset_confirm'),
    
    # Facebook auth.
    url(r'^/login/facebook/?$', views_facebook.login_request, name='facebook_login'),
    url(r'^/login/facebook/return/?$', views_facebook.login_return, name='facebook_return'),
    url(r'^/create/facebook/?$', views_facebook.create_account, name='facebook_create'),
    url(r'^/disconnect/facebook/?$', views_facebook.disconnect_account, name='facebook_disconnect'),
    
    # Twitter auth.
    url(r'^/login/twitter/?$', views_twitter.login_request, name='twitter_login'),
    url(r'^/login/twitter/return/?$', views_twitter.login_return, name='twitter_return'),
    url(r'^/create/twitter/?$', views_twitter.create_account, name='twitter_create'),
    url(r'^/disconnect/twitter/?$', views_twitter.disconnect_account, name='twitter_disconnect')
)


import datetime
import json
import logging
import oauth2 as oauth
import urlparse
import twitter
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login, get_user_model
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.core.validators import validate_email
from django.db.utils import IntegrityError
from django.http.response import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.utils.translation import ugettext as _
from yapi.utils import generate_key

from models import AuthenticationLog, TwitterProfile

# Instantiate logger.
logger = logging.getLogger(__name__)

# Application's OAuth settings.
consumer_key = settings.YACCOUNTS['twitter_oauth']['consumer_key']
consumer_secret = settings.YACCOUNTS['twitter_oauth']['consumer_secret']

# Twitter OAuth URLs.
request_token_url = 'https://api.twitter.com/oauth/request_token'
authorize_url = 'https://api.twitter.com/oauth/authorize'
access_token_url = 'https://api.twitter.com/oauth/access_token'

# Init Twitter API.
consumer = oauth.Consumer(consumer_key, consumer_secret)
client = oauth.Client(consumer)


class UserInfo:
    """
    Twitter userinfo wrapper.
    (if necessary to have an object similar to what is returned by the twitter lib)
    """
    def __init__(self, twitter_user_id, screen_name):
        self.id = twitter_user_id
        self.screen_name = screen_name


def login_request(request):
    """
    Starts Twitter authentication.
    """
    
    # Check if Twitter authentication is enabled.
    try:
        settings.YACCOUNTS['signup_available'].index('TWITTER')
    except ValueError:
        messages.add_message(request, messages.ERROR, _('Twitter login not available.'))
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    # If there is an URL to return when login finishes,
    # store it in session in order to make it acessible in
    # the twitter return method (would be better to pass it in the URLs?)
    request.session['login_next'] = request.GET.get('next', reverse('yaccounts:index'))
    
    # Step 1: Get a request token. This is a temporary token that is used for 
    # having the user authorize an access token and to sign the request to obtain 
    # said access token.
    oauth_callback = settings.HOST_URL + reverse('yaccounts:twitter_return')
    resp, content = client.request(request_token_url + '?oauth_callback=' + oauth_callback, 'GET')
    if resp['status'] != '200':
        logger.error('Twitter Login Error: unable to request token. ' + content)
        raise Exception("Invalid response %s." % resp['status'])
    request_token = dict(urlparse.parse_qsl(content))
    
    # Add request token to Django session in order to be accessible on oauth callback.
    request.session['request_token'] = request_token
    
    # Step 2: Redirect to the provider.
    return HttpResponseRedirect("%s?oauth_token=%s" % (authorize_url, request_token['oauth_token']))
    
    
def login_return(request):
    """
    OAuth callback.
    """
    
    # If there is no request_token for session,
    # it means we didn't redirect user to Twitter.
    request_token = request.session.get('request_token', None)
    if not request_token:
        messages.add_message(request, messages.ERROR, _('Twitter login error #1'))
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    # If the token from session and token from Twitter does not match,
    # it means something bad happened to tokens.
    elif request_token['oauth_token'] != request.GET.get('oauth_token', None):
        messages.add_message(request, messages.ERROR, _('Twitter login error #2'))
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    
    # Now that we're here, we don't need this in session variables anymore. Cleanup.
    else:
        del request.session['request_token']
    
    # Step 3: Once the consumer has redirected the user back to the oauth_callback
    # URL you can request the access token the user has approved. You use the 
    # request token to sign this request. After this is done you throw away the
    # request token and use the access token returned. You should store this 
    # access token somewhere safe, like a database, for future use.
    token = oauth.Token(request_token['oauth_token'], request_token['oauth_token_secret'])
    token.set_verifier(request.GET['oauth_verifier'])
    client = oauth.Client(consumer, token)
    
    resp, content = client.request(access_token_url, 'POST')
    access_token = dict(urlparse.parse_qsl(content))
    
    # Verify credentials.
    try:
        api = twitter.Api(consumer_key=consumer_key,
                          consumer_secret=consumer_secret,
                          access_token_key=access_token['oauth_token'],
                          access_token_secret=access_token['oauth_token_secret'])
        userinfo = api.VerifyCredentials()
    except twitter.TwitterError:
        messages.add_message(request, messages.ERROR, _('Twitter login error #3'))
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    #
    # Finally, authenticate user with given Twitter credentials.
    #
    user = authenticate(twitter_userinfo=userinfo, twitter_access_token=access_token)
    
    ##
    # a) Twitter profile is linked with existing user account.
    if user:
        
        # Additional information on the request that should be logged.
        metadata = { 'user_agent': request.META['HTTP_USER_AGENT'] }
        
        # If user is active, login account and redirect to next page (if provided, else account profile)
        if user.is_active:
            
            # Create user login session.
            login(request, user)
            
            # Log authentication.
            AuthenticationLog.new(email=user.email,
                                  valid_credentials=True,
                                  credentials_type='twitter',
                                  account_status='active',
                                  success=True,
                                  ip_address=request.META['REMOTE_ADDR'],
                                  metadata=json.dumps(metadata))
            
            # Redirect to either page referenced as next or accounts index.
            return HttpResponseRedirect(request.session.get('login_next', reverse('yaccounts:index')))
        
        # User account is inactive.
        else:
            
            # Log authentication.
            AuthenticationLog.new(email=user.email,
                                  valid_credentials=True,
                                  credentials_type='twitter',
                                  account_status='disabled',
                                  success=False,
                                  ip_address=request.META['REMOTE_ADDR'],
                                  metadata=json.dumps(metadata))
            
            # Message.
            messages.warning(request, _("Your account is disabled."))
            
            # Return.
            return HttpResponseRedirect(reverse('yaccounts:login'))
        
    ##
    # b) Unknown Twitter profile.
    else:
        
        # i) If there is an account logged in, (attempt to) link the Twitter profile with it.
        if request.user.is_authenticated():
            try:
                TwitterProfile.new(user=request.user, userinfo=userinfo, access_token=access_token)
                messages.success(request, _("Twitter account connected successfully."))
            except IntegrityError:
                messages.error(request, _("You already have a Twitter profile linked to your account."))
            return HttpResponseRedirect(reverse('yaccounts:index'))
        
        # ii) Create new account.
        else:
            # Place Twitter info in a session variable in order for it to be accessed in the registration page.
            request.session['twitter_create'] = {
                    'twitter_user_id': userinfo.id,
                    'screen_name': userinfo.screen_name,
                    'profile_image_url': userinfo.profile_image_url,
                    'name': userinfo.name,
                    'access_token': access_token,
                    'expires': (datetime.datetime.now() + datetime.timedelta(seconds=5*60)).strftime('%s') # Convert to epoch to be JSON serializable.
            }
            return HttpResponseRedirect(reverse('yaccounts:twitter_create'))


def create_account(request):
    """
    Create new account with Twitter credentials.
    """
    
    #
    # Lets validate if we should be here.
    #
    
    # If user is authenticated, then this place shouldn't be reached.
    # Twitter profile, if valid, should be linked with the logged in account and not create a new account.
    if request.user.is_authenticated():
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    # In order to create account with Twitter profile, its details should have been stored in session.
    twitter_create = request.session.get('twitter_create', None)
    if not twitter_create:
        messages.error(request, _('Twitter login error #4'))
        return HttpResponseRedirect(reverse('yaccounts:login'))
    
    # If time window for registration of new account with this Twitter profile has expired,
    # delete it from session and restart Twitter login process.
    if datetime.datetime.now() > datetime.datetime.fromtimestamp(float(twitter_create['expires'])):
        del request.session['twitter_create']
        return HttpResponseRedirect(reverse('yaccounts:twitter_login'))
    
    #
    # Proceed with account creation.
    #
    email = ''
    
    # A form was received.
    if request.method == 'POST':
        
        proceed = True
        
        # Fetch mandatory params.
        try:
            email = request.POST['email']
        except KeyError:
            proceed = False
            messages.error(request, _("Please provide an e-mail address."))
            
        # Validate email address.
        try:
            validate_email(email)
        except ValidationError:
            proceed = False
            messages.error(request, _("Please provide a valid email address."))
            
        # Check if Twitter profile is already connected to another account.
        try:
            TwitterProfile.objects.get(twitter_user_id=twitter_create['twitter_user_id'])
            proceed = False
            messages.error(request, _("Twitter profile already connected to another account."))
        except ObjectDoesNotExist:
            pass
        
        # Check if account exists with given email address.
        try:
            get_user_model().objects.get(email=email)
            proceed = False
            messages.error(request, _("Email already registered."))
        except ObjectDoesNotExist:
            pass
        
        #
        # Everything checks! \o/
        #
        if proceed:
            
            # 1) Create user with random password.
            try:
                random_password = generate_key(email + str(datetime.datetime.now()) + str(twitter_create['twitter_user_id']))
                random_password += datetime.datetime.now().strftime('%s')
                user = get_user_model().new(name=twitter_create['name'], email=email, password=random_password, credentials_type='twitter')
            except:
                logger.error('Error creating user via Twitter! #5 ' + str(twitter_create), exc_info=1)
                messages.error(request, _('Twitter login error #5'))
            
            # 2) Create Twitter profile and associate it with the new user.
            try:
                userinfo = UserInfo(twitter_user_id=twitter_create['twitter_user_id'], screen_name=twitter_create['screen_name'])
                TwitterProfile.new(user=user, userinfo=userinfo, access_token=twitter_create['access_token'])
                # Redirect to login page with message.
                messages.success(request, _("An email was sent in order to confirm your account."))
                return HttpResponseRedirect(reverse('yaccounts:login'))
            except:
                user.delete() # Delete newly created user (as it would be inaccessible since the Twitter Profile wasn't created!)
                logger.error('Error creating user via Twitter! #6 ' + str(twitter_create), exc_info=1)
                messages.error(request, _('Twitter login error #6'))
    
    # Render page.
    return render_to_response('yaccounts/create_social.html',
                              { 'avatar': twitter_create['profile_image_url'],
                               'username': '@' + twitter_create['screen_name'],
                               'email': email,
                               'post_url': reverse('yaccounts:twitter_create') },
                              context_instance=RequestContext(request))
    
    
@login_required
def disconnect_account(request):
    """
    Disconnects Twitter profile from user account.
    """
    
    # Check if user has a Twitter profile connected.
    if not hasattr(request.user, 'twitterprofile'):
        messages.error(request, _("You don't have a Twitter account connected."))
    
    # Disconnect!
    else:
        request.user.twitterprofile.delete()
        messages.success(request, _("Twitter account disconnected."))
    
    # Redirect to account page.
    return HttpResponseRedirect(reverse('yaccounts:index'))


import datetime
import json
import logging
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login, logout, get_user_model
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.http.response import HttpResponseRedirect, Http404
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from exceptions import InvalidParameter
from models import AuthenticationLog, ResetRequest, EmailUpdate
from utils import ConfirmationToken

# Instantiate logger.
logger = logging.getLogger(__name__)


@login_required
def index(request):
    """
    Account index.
    """
        
    # If user has Twitter profile.
    if hasattr(request.user, 'twitterprofile'):
        twitter_profile = request.user.twitterprofile
    else:
        twitter_profile = None
        
    # If user has Facebook profile.
    if hasattr(request.user, 'facebookprofile'):
        facebook_profile = request.user.facebookprofile
    else:
        facebook_profile = None
    
    # Render page.
    return render_to_response('yaccounts/index.html',
                              { 'name': request.user.name,
                                'email': request.user.email,
                                'photo_url': request.user.get_photo_url(),
                                'photo_update_api_url': settings.HOST_URL + reverse(settings.YACCOUNTS['api_url_namespace'] + ':yaccounts:photo'),
                                'account_update_api_url': settings.HOST_URL + reverse(settings.YACCOUNTS['api_url_namespace'] + ':yaccounts:index'),
                                'api_keys_api_url': settings.HOST_URL + reverse(settings.YACCOUNTS['api_url_namespace'] + ':yaccounts:api_keys'),
                                'twitter_profile': twitter_profile, 'facebook_profile': facebook_profile },
                              context_instance=RequestContext(request))


def login_account(request):
    """
    Login account.
    """
    # If a user is logged in, redirect to account page.
    if request.user.is_authenticated():
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    ################################################################
    #                     A form was submitted.                    #
    ################################################################
    if request.method == 'POST':
        
        # Additional information on the request that should be logged.
        metadata = { 'user_agent': request.META['HTTP_USER_AGENT'] }
        
        #
        # 1) Fetch params.
        #
        email = request.POST.get('email', '')
        password = request.POST.get('password', '')
        remember = request.POST.get('remember', None)
        
        # Mandatory parameters not provided.
        if email == '' or password == '':
            # Message.
            messages.error(request, _("Please provide your email and password."))
        
        #
        # 2) Validate credentials.
        #
        else:
            
            user = authenticate(email=email, password=password)
            
            ######################
            # VALID credentials. #
            ######################
            if user is not None:
                
                #
                # a) Account is ACTIVE.
                #
                if user.is_active:
                    
                    # Login user.
                    login(request, user)
                    
                    # If NOT flaged to remember.
                    if not remember: 
                        request.session.set_expiry(0)
                        
                    # Log authentication.
                    AuthenticationLog.new(email=email,
                                          valid_credentials=True,
                                          credentials_type='email',
                                          account_status='active',
                                          success=True,
                                          ip_address=request.META['REMOTE_ADDR'],
                                          metadata=json.dumps(metadata))
                    
                    #
                    # *** If login successful, REDIRECT to profile page or to provided url. ***
                    #
                    return HttpResponseRedirect(request.POST.get('next', reverse('yaccounts:index')))
                
                #
                # b) Account is PENDING ACTIVATION.
                #
                elif user.pending_activation():
                    
                    # Log authentication.
                    AuthenticationLog.new(email=email,
                                          valid_credentials=True,
                                          credentials_type='email',
                                          account_status='pending_activation',
                                          success=False,
                                          ip_address=request.META['REMOTE_ADDR'],
                                          metadata=json.dumps(metadata))
                    
                    # Message.
                    messages.warning(request, mark_safe(_("You didn't activate your account. <a href=\"\">Resend confirmation email</a>.")))
                
                #
                # c) Account is DISABLED.
                #
                else:
                    
                    # Log authentication.
                    AuthenticationLog.new(email=email,
                                          valid_credentials=True,
                                          credentials_type='email',
                                          account_status='disabled',
                                          success=False,
                                          ip_address=request.META['REMOTE_ADDR'],
                                          metadata=json.dumps(metadata))
                    
                    # Message.
                    messages.warning(request, _("Your account is disabled."))
            
            ########################
            # INVALID credentials. #
            ########################
            else:
                
                # Check if account with given email address exists.
                try:
                    user = get_user_model().objects.get(email=email)
                    # Account exists, check status.
                    if user.is_active:
                        account_status = 'active'
                    elif user.pending_activation():
                        account_status = 'pending_activation'
                    else:
                        account_status = 'disabled'
                # There is no account with given email address.
                except ObjectDoesNotExist:
                    account_status = 'does_not_exist'
                
                # Log authentication.
                AuthenticationLog.new(email=email,
                                      valid_credentials=False,
                                      credentials_type='email',
                                      account_status=account_status,
                                      success=False,
                                      ip_address=request.META['REMOTE_ADDR'],
                                      metadata=json.dumps(metadata))
                
                # Message.
                messages.error(request, _("Invalid credentials."))
        
        #
        # 3) If this place is reached, then login was unsuccessful.
        # The login page is rendered with respective message and pre-filled fields.
        #
        return render_to_response('yaccounts/login.html',
                                  { 'email': email, 'next': request.POST.get('next', None) },
                                  context_instance=RequestContext(request))
    
    ################################################################
    #                      Render login page.                      #
    ################################################################
    else:
        return render_to_response('yaccounts/login.html',
                                  { 'next': request.GET.get('next', None) },
                                  context_instance=RequestContext(request))


def logout_account(request):
    """
    Logout.
    """
    if request.user.is_authenticated():
        logout(request)
    return HttpResponseRedirect(request.GET.get('next', reverse('yaccounts:index')))


def create_account(request):
    """
    Creates a new account.
    """
    # If a user is logged in, redirect to account page.
    if request.user.is_authenticated():
        return HttpResponseRedirect(reverse('yaccounts:index'))
    
    ################################################################
    #                     A form was submitted.                    #
    ################################################################
    if request.method == 'POST':
        
        # Check if this feature is enabled.
        try:
            settings.YACCOUNTS['signup_available'].index('EMAIL')
        except:
            raise Http404
        
        #
        # 1) Fetch params.
        #
        name = request.POST.get('name', '')
        email = request.POST.get('email', '')
        password = request.POST.get('password', '')
        
        # Mandatory parameters not provided.
        if name == '' or email == '' or password == '':
            messages.error(request, _("Please provide your name, email and a password."))
        
        #
        # 2) Create account.
        #
        else:
            
            # If all parameters check, then a new account is created.
            try:
                get_user_model().new(name=name,
                                     email=email,
                                     password=password,
                                     credentials_type='email')
                messages.success(request, _("An email was sent in order to confirm your account."))
                return HttpResponseRedirect(reverse('yaccounts:login'))
            
            # Invalid parameters.
            except InvalidParameter as e:
                messages.error(request, e.message)    
            
            # Unknown error.
            except:
                messages.error(request, _("Error creating account, please contact support."))
        
        #
        # 3) If this place is reached, then account creation form contained errors.
        # The sign-up page is rendered with respective message and pre-filled fields.
        #
        return render_to_response('yaccounts/create.html',
                                  { 'name': name, 'email': email, 'next': request.POST.get('next', None) },
                                  context_instance=RequestContext(request))
    
    ################################################################
    #                     Render sign-up page.                     #
    ################################################################
    else:
        return render_to_response('yaccounts/create.html',
                                  { 'next': request.GET.get('next', None) },
                                  context_instance=RequestContext(request))


def confirm_operation(request):
    """
    Confirm operation (e.g. Account activation, email change, etc)
    """
    #
    # Fetch token.
    #
    try:
        token = request.GET['t']
        if token == '':
            raise KeyError
    except KeyError:
        raise Http404
    
    #
    # Process token.
    #
    processed_token = ConfirmationToken.process(token)
    
    # Invalid token.
    if not processed_token:
        raise Http404
    
    # Token info.
    else:
        operation = processed_token['operation']
        email = processed_token['email']
        key = processed_token['key']
    
    #
    # Validate operation.
    #
    
    ## Account activation.
    #
    if operation == ConfirmationToken.ACCOUNT_ACTIVATION:
        
        # Verify activation key, by attempting to authenticate user using it.
        user = authenticate(email=email, activation_key=key)
        
        # If a user is returned, then the activation key checked out and the account activated.
        if user is not None:
            
            # Login user.
            login(request, user)
            
            # Redirect to account page.
            return HttpResponseRedirect(reverse('yaccounts:index'))
            
        # Invalid activation key.
        else:
            logger.info('Invalid activation key! Data: ' + str(processed_token) + ', IP Address: ' + request.META['REMOTE_ADDR'])
            raise Http404
        
    ## Email update.
    #
    elif operation == ConfirmationToken.ACCOUNT_EMAIL_UPDATE:
        
        # Fetch additional required params.
        try:
            new_email = processed_token['new_email']
        except KeyError:
            logger.error('Error updating email! Missing new_email. Data: ' + str(processed_token) + ', IP Address: ' + request.META['REMOTE_ADDR'])
            raise Http404
        
        # Check if user with given email exists.
        try:
            user = get_user_model().objects.get(email=email)
        except ObjectDoesNotExist:
            logger.error('Error updating email! User does not exist. Data: ' + str(processed_token) + ', IP Address: ' + request.META['REMOTE_ADDR'])
            raise Http404
        
        # Check if there is a pending update request for the user->email->key.
        email_update = EmailUpdate.objects.filter(user=user, new_email=new_email, key=key, updated_at__isnull=True)
        if not email_update:
            logger.error('Error updating email! No pending update for given details. Data: ' + str(processed_token) + ', IP Address: ' + request.META['REMOTE_ADDR'])
            raise Http404
        
        # GREAT SUCCESS!
        else:
            
            # Update email.
            user.update(email=new_email)
            
            # Mark key as used.
            for update in email_update:
                update.updated_at = datetime.datetime.now()
                update.save()
            
            # Message.
            messages.success(request, _("Your new email was confirmed."))
            
            # Redirect to account page.
            return HttpResponseRedirect(reverse('yaccounts:index'))
    
    ## Invalid operation.
    #
    else:
        logger.error('Invalid account confirmation OPERATION! Data: ' + str(processed_token) + ', IP Address: ' + request.META['REMOTE_ADDR'])
        raise Http404


def reset_account(request):
    """
    Request password reset.
    """
    
    email = ''
    
    # A reset was requested.
    if request.method == 'POST':
        
        # Mandatory params.
        email = request.POST.get('email', '')
        if email == '':
            messages.error(request, _("Please provide your email."))
        
        # Business as usual...
        else:
            # Check if user with given email exists.
            try:
                user = get_user_model().objects.get(email=email)
                
                # Check if account is active.
                if not user.is_active:
                    messages.error(request, _("Your account is disabled."))
                
                #
                # If this place is reached, then we have a valid account. Proceed with reset request! ------------->
                #
                
                # Check if a password reset for this user wasn't requested less than 5 mins ago.
                elif ResetRequest.objects.filter(user=user,
                                               created_at__gte=(datetime.datetime.now() - datetime.timedelta(0, 300))):
                    messages.error(request, _("A reset was requested recently."))
                    
                # Send reset email.
                else:
                    try:
                        ResetRequest.new(user)
                        email = ''
                        messages.success(request, _("An email was sent with reset instructions."))
                    except:
                        logger.error('Unable to reset password! Email: ' + email, exc_info=1)
                        messages.error(request, _("Unable to reset password."))
                
                #
                # <-------------------------------------------------------------------------------------------------
                #
                
            except ObjectDoesNotExist:
                messages.error(request, _("User does not exist."))
    
    # Render page.
    return render_to_response('yaccounts/reset.html',
                              { 'email': email },
                              context_instance=RequestContext(request))
    

def reset_confirm(request):
    """
    If token is valid, allow password change.
    """
    
    #
    # Process token.
    #
    try:
        token = request.GET['t']
        processed_token = ConfirmationToken.process(token)
    except KeyError:
        raise Http404
    
    # Invalid token.
    if not processed_token:
        raise Http404
    
    # Token info.
    else:
        operation = processed_token['operation']
        email = processed_token['email']
        key = processed_token['key']
        
    #
    # Validate email.
    #
    try:
        user = get_user_model().objects.get(email=email)
    except ObjectDoesNotExist:
        logger.warning('Valid reset token for an invalid user! Email: ' + email + ', Key: ' + key + ', Operation: '+ operation + ', IP Address: ' + request.META['REMOTE_ADDR'])
        raise Http404
    
    #
    # Validate operation.
    #
    if operation == ConfirmationToken.ACCOUNT_PASSWORD_RESET:
        
        # Check if an unused reset key exists.
        try:
            reset_request = ResetRequest.objects.get(user=user, key=key, reset_at__isnull=True)
        except ObjectDoesNotExist:
            logger.debug('Invalid reset key! Email: ' + email + ', Key: ' + key)
            raise Http404
        
        # A password update was requested.
        if request.method == 'POST':
            
            # Mandatory params.
            password = request.POST.get('password', '')
            password_confirm = request.POST.get('password_confirm', '')
            if password == '' or password_confirm == '':
                messages.error(request, _("Please provide a password and respective confirmation."))
                
            # Check if new password and confirmation match.
            elif password != password_confirm:
                messages.error(request, _("Passwords don't match."))
        
            # Proceed with reset!
            else:
                
                # Mark key as used.
                reset_request.reset_at = datetime.datetime.now()
                reset_request.save()
                
                # Update user's password.
                reset_request.user.update(password=password)
                
                # Login user.
                user = authenticate(email=email, password=password)
                login(request, user)
                
                # Redirect to account page.
                messages.success(request, _("Your password was updated."))
                return HttpResponseRedirect(reverse('yaccounts:index'))
        
        # Render page.
        return render_to_response('yaccounts/reset_confirm.html',
                                  { 'token': token },
                                  context_instance=RequestContext(request))
    
    # Invalid operation.
    else:
        logger.info('Invalid reset confirmation OPERATION! Email: ' + email + ', Key: ' + key + ', Operation: ' + operation + ', IP Address: ' + request.META['REMOTE_ADDR'])
        raise Http404
    


import logging
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q
from django.http.response import HttpResponse
from yapi.models import ApiKey
from yapi.authentication import SessionAuthentication, ApiKeyAuthentication
from yapi.resource import Resource
from yapi.response import HTTPStatus, Response
from yapi.utils import generate_key

from serializers import UserSerializer, UserPhotoSerializer, ApiKeySerializer
from yaccounts.forms import UserForm, UserPhotoForm

# Instantiate logger.
logger = logging.getLogger(__name__)


class AccountHandler(Resource):
    """
    API endpoint handler.
    """
    # HTTP methods allowed.
    allowed_methods = ['GET', 'PUT']
    
    # Authentication & Authorization.
    authentication = [SessionAuthentication, ApiKeyAuthentication]
    
    def get(self, request):
        """
        Process GET request.
        """
        return Response(request=request,
                        data=request.auth['user'],
                        serializer=UserSerializer,
                        status=HTTPStatus.SUCCESS_200_OK)
        
    def put(self, request):
        """
        Process PUT request.
        """
        # Populate form with provided data.
        form = UserForm(request.data, user=request.auth['user'])
        
        # Provided data is valid.
        if form.is_valid():
            
            # Update account.
            request.auth['user'].update(name=form.cleaned_data['name'],
                                        password=form.cleaned_data['password_new'])
            
            # Return.
            return Response(request=request,
                            data=request.auth['user'],
                            serializer=UserSerializer,
                            status=HTTPStatus.SUCCESS_200_OK)
            
        # Form didn't validate!
        else:
            return Response(request=request,
                            data={ 'message': 'Invalid parameters', 'parameters': form.errors },
                            serializer=None,
                            status=HTTPStatus.CLIENT_ERROR_400_BAD_REQUEST)
        
        
class AccountPhotoHandler(Resource):
    """
    API endpoint handler.
    """
    # HTTP methods allowed.
    allowed_methods = ['POST', 'GET']
    
    # Authentication & Authorization.
    authentication = [SessionAuthentication, ApiKeyAuthentication]
    
    def get(self, request):
        """
        Process GET request.
        """
        # If user has photo.
        if hasattr(request.auth['user'], 'userphoto'):
            return Response(request=request,
                            data=request.auth['user'].userphoto,
                            serializer=UserPhotoSerializer,
                            status=HTTPStatus.SUCCESS_200_OK)
         
        # No photozzz!
        else:
            return HttpResponse(status=HTTPStatus.CLIENT_ERROR_404_NOT_FOUND)

    def post(self, request):
        """
        Process POST request.
        """
        # Populate form with provided data.
        form = UserPhotoForm(request.POST, request.FILES)
        
        # Provided data is valid.
        if form.is_valid():
            
            # Update account photo and return.
            return Response(request=request,
                            data=request.auth['user'].set_photo(form.cleaned_data['file']),
                            serializer=UserPhotoSerializer,
                            status=HTTPStatus.SUCCESS_201_CREATED)
            
        # Form didn't validate!
        else:
            return Response(request=request,
                            data={ 'message': 'Invalid parameters', 'parameters': form.errors },
                            serializer=None,
                            status=HTTPStatus.CLIENT_ERROR_400_BAD_REQUEST)


class ApiKeysHandler(Resource):
    """
    API endpoint handler.
    """
    # HTTP methods allowed.
    allowed_methods = ['POST', 'GET', 'PUT']
    
    # Authentication & Authorization.
    authentication = [SessionAuthentication, ApiKeyAuthentication]
    
    def post(self, request):
        """
        Process POST request.
        """
        # Validate parameters.
        try:
            description = request.data['description']
        except KeyError:
            return Response(request=request,
                            data={ 'message': 'Missing parameter: description' },
                            serializer=None,
                            status=HTTPStatus.CLIENT_ERROR_400_BAD_REQUEST)
            
        # Create API key.
        api_key = ApiKey(user=request.auth['user'],
                        key=generate_key(request.auth['user'].email),
                        description=description,
                        active=True)
        api_key.save()
        
        # Return.
        return Response(request=request,
                        data=api_key,
                        serializer=ApiKeySerializer,
                        status=HTTPStatus.SUCCESS_201_CREATED)
        
    def get(self, request):
        """
        Process GET request.
        """
        #
        # Lets start with all.
        #
        results = ApiKey.objects.filter(user=request.auth['user']).order_by('created_at')
        
        #
        # Filters
        #
        filters = {}
        
        # Search description.
        try:
            description = request.GET['description']
            if description != '':
                results = results.filter(Q(description__icontains=description))
                filters['description'] = description
        except KeyError:
            pass
        
        # Status.
        try:
            active = request.GET['active']
            if active != '':
                # Validate parameter.
                if active.lower() != 'true' and active.lower() != 'false':
                    return Response(request=request,
                            data={ 'message': 'Invalid parameter value', 'param': 'active' },
                            serializer=None,
                            status=HTTPStatus.CLIENT_ERROR_400_BAD_REQUEST)
                # Business as usual...
                else:
                    is_active = active.lower() == 'true'
                    results = results.filter(active=is_active)
                    filters['active'] = is_active
        except KeyError:
            pass
        
        #
        # Return.
        #
        return Response(request=request,
                        data=results,
                        filters=filters,
                        serializer=ApiKeySerializer,
                        pagination=False,
                        status=HTTPStatus.SUCCESS_200_OK)


class ApiKeyIdHandler(Resource):
    """
    API endpoint handler.
    """
    # HTTP methods allowed.
    allowed_methods = ['GET', 'PUT', 'DELETE']
    
    # Authentication & Authorization.
    authentication = [SessionAuthentication, ApiKeyAuthentication]
    
    def get(self, request, pk):
        """
        Process GET request.
        """
        # Check if API key with given ID exists for given user.
        try:
            api_key = ApiKey.objects.get(user=request.auth['user'], id=pk)
        except ObjectDoesNotExist:
            return HttpResponse(status=HTTPStatus.CLIENT_ERROR_404_NOT_FOUND)
        
        # Return.
        return Response(request=request,
                        data=api_key,
                        serializer=ApiKeySerializer,
                        status=HTTPStatus.SUCCESS_200_OK)
        
    def put(self, request, pk):
        """
        Process PUT request.
        """
        # Check if API key with given ID exists for given user.
        try:
            api_key = ApiKey.objects.get(user=request.auth['user'], id=pk)
        except ObjectDoesNotExist:
            return HttpResponse(status=HTTPStatus.CLIENT_ERROR_404_NOT_FOUND)
        
        # Optional params.
        try:
            description = request.data['description']
        except KeyError:
            description = None
        try:
            active = request.data['active']
            # Validate param.
            if active == '' or (active != True and active != False):
                return Response(request=request,
                            data={ 'message': 'Invalid parameter value', 'param': 'active' },
                            serializer=None,
                            status=HTTPStatus.CLIENT_ERROR_400_BAD_REQUEST)
        except:
            active = None
            
        # Update.
        if description:
            api_key.description = description
        if active == True or active == False:
            api_key.active = active
        api_key.save()
        
        # Return.
        return HttpResponse(status=HTTPStatus.SUCCESS_200_OK)
        
    def delete(self, request, pk):
        """
        Process DELETE request.
        """
        # Check if API key with given ID exists for given user.
        try:
            api_key = ApiKey.objects.get(user=request.auth['user'], id=pk)
        except ObjectDoesNotExist:
            return HttpResponse(status=HTTPStatus.CLIENT_ERROR_404_NOT_FOUND)
        
        # Delete API key.
        api_key.delete()
        
        # Return.
        return HttpResponse(status=HTTPStatus.SUCCESS_204_NO_CONTENT)


import base64
import json
import logging
import types

# Instantiate logger.
logger = logging.getLogger(__name__)


class ConfirmationToken:
    """
    Response status codes.
    """
    
    # Available operations.
    ACCOUNT_ACTIVATION = 'activation'
    ACCOUNT_PASSWORD_RESET = 'password_reset'
    ACCOUNT_EMAIL_UPDATE = 'email_update'
    
    @staticmethod
    def generate(email, operation, key, more=None):
            """
            Returns a Base64 encoded string containing the confirmation data.
            """
            # Core token info.
            token = {
                'email': email,
                'operation': operation,
                'key': key
            }
            
            # Additional information.
            if more:
                if not isinstance(more, types.DictType):
                    raise TypeError("Expecting a dict")
                token.update(more)
                
            # Return.
            return base64.b64encode(json.dumps(token))
        
    @staticmethod
    def process(token):
        """
        Process confirmation token and, if valid, extract respective info.
        """
        try:
            # A valid token is a base64 encoded string.
            confirm_data = json.loads(base64.b64decode(token))
            
            # Containing the account's email, confirmation scenario and respective key.
            try:
                confirm_data['email']
                confirm_data['operation']
                confirm_data['key']
                
                # If this place is reached, then the token is valid. Return respective info.
                return confirm_data
                
            except KeyError:
                logger.info('Invalid account confirmation DATA! Data: ' + json.dumps(confirm_data))
                return None
        
        # Unable to b64 decode.
        except TypeError:
            logger.info('Invalid BASE64 account confirmation token! Token: ' + token)
            return None
        
        # Error decoding JSON
        except ValueError:
            logger.info('Invalid JSON account confirmation token! Token: ' + token)
            return None


import logging
from django.conf import settings
from django.core.urlresolvers import reverse
from yapi.serializers import BaseSerializer

# Instantiate logger.
logger = logging.getLogger(__name__)


class UserSerializer(BaseSerializer):
    """
    Adds methods required for instance serialization.
    """
        
    def to_simple(self, obj, user=None):
        """
        Please refer to the interface documentation.
        """
        # Build response.
        simple = {
            'email': obj.email,
            'name': obj.name,
            'last_login': obj.last_login.strftime("%Y-%m-%d %H:%M:%S"),
            'photo': {
                'url': settings.HOST_URL + reverse(settings.YACCOUNTS['api_url_namespace'] + ':yaccounts:photo'),
                'image_url': obj.get_photo_url()
            },
            'api_keys': {
                'url': settings.HOST_URL + reverse(settings.YACCOUNTS['api_url_namespace'] + ':yaccounts:api_keys')
            }
        }
        
        # Twitter Profile.
        if hasattr(obj, 'twitterprofile'):
            simple['twitter_profile'] = {
                'twitter_user_id': obj.twitterprofile.twitter_user_id,
                'screen_name': obj.twitterprofile.screen_name,
                'profile_link': 'http://twitter.com/' + obj.twitterprofile.screen_name
            }
        
        # Facebook Profile.
        if hasattr(obj, 'facebookprofile'):
            simple['facebook_profile'] = {
                'facebook_user_id': obj.facebookprofile.facebook_user_id,
                'name': obj.facebookprofile.name,
                'profile_link': 'http://facebook.com/profile.php?id=' + obj.facebookprofile.facebook_user_id,
                'profile_image_url': obj.facebookprofile.get_photo_url()
            }
        
        # Return.
        return simple
    
    
class UserPhotoSerializer(BaseSerializer):
    """
    Adds methods required for instance serialization.
    """
    
    def to_simple(self, obj, user=None):
        """
        Please refer to the interface documentation.
        """
        simple = {
            'image_url': obj.file.url
        }        
        return simple
    
    
class ApiKeySerializer(BaseSerializer):
    """
    Adds methods required for instance serialization.
    """
    
    def to_simple(self, obj, user=None):
        """
        Please refer to the interface documentation.
        """
        # Build response.
        simple = {
            'id': obj.id,
            'url': settings.HOST_URL + reverse(settings.YACCOUNTS['api_url_namespace'] + ':yaccounts:api_key_id', args=[obj.id]),
            'key': obj.key,
            'description': obj.description,
            'created_at': obj.created_at.strftime("%Y-%m-%d %H:%M:%S"),
            'active': obj.active
        }
        if obj.last_used:
            simple['last_used'] = obj.last_used.strftime("%Y-%m-%d %H:%M:%S")
        
        # Return.
        return simple


#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-yaccounts',
    version='0.4',
    packages=['yaccounts'],
    include_package_data=True,
    license='BSD License',  # example license
    description='Accounts app.',
    #long_description=README,
    #url='http://www.example.com/',
    author='André Tavares',
    author_email='github@andretavares.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License', # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'PIL==1.1.7',
        'python-twitter==1.1',
        'facebook-sdk==0.4.0',
        'django-yapi',
        'django-yutils'
    ]
)